#include <cstdio>
#include <cstdlib>

#include "db.hh"

using namespace std;

db::db() : status(0)
{
	mysql_init(&sql);
}

db::db(const string &host,
	   const string &user,
	   const string &pass)
{
	status = 0;
    mysql_init(&sql);
    connect(host, user, pass);
}
	
void db::connect(const string &host,
	             const string &user,
	             const string &pass)
{
    mysql_connect(&sql, host.c_str(), user.c_str(), pass.c_str());
}

db::~db()
{
	mysql_close(&sql);
}

db::result db::query(const string &s)
{
//	cerr << "Issuing db query " << s << " (" << s.size() << " bytes)\n";
	int tmp = mysql_real_query(&sql, s.data(), s.size());
//	cerr << "Query returned " << tmp << endl;
	if(tmp) return result(NULL);
	
	MYSQL_RES *r = mysql_use_result(&sql);
	result res(r);
	res.movenext();
	
	return res;
}

db::result::result(MYSQL_RES *r) : res(r), isok(false)
{
	if(!r)
	{
		refcount = NULL;
		return;
	}
	
	refcount = new int(1);
	
	fieldcount = mysql_field_count(res);
	fields.resize(fieldcount);
	
	MYSQL_FIELD *flds = mysql_fetch_fields(res);
	
	for(unsigned a=0; a<fieldcount; ++a)
		fields[a] = flds[a].name;
}

db::result::result(const result &r) : rowtype(r)
{
	*this = r;
	if(refcount)++*refcount;
}

db::result::~result()
{
	if(refcount)
	{
		if(!res)
			cerr << "voicebox: REFCOUNT SET WITHOUT MYSQL RESULT?? - THIS IS A BUG.\n";
		else if(!--*refcount)
		{
			delete refcount;
			mysql_free_result(res);
		}		
	}
	else if(res)
		cerr << "voicebox: MYSQL RESULT DESTRUCTED WITHOUT REFCOUNT - THIS IS A BUG.\n";
}

void db::result::genmap()
{
	clear();
	for(unsigned a=0; a<fieldcount; ++a)
	{
		const char *value = row[a];
		
		if(value)
		{
			string s(value, lens[a]);
			(*this) [fields[a]] = s;
		}
		else
			(*this) [fields[a]] = "";		
	}
}

bool db::result::movenext()
{
	if(!res)return (isok = false);
	
	row = mysql_fetch_row(res);
	if(row)
	{
		lens = mysql_fetch_lengths(res);
		isok = true;
		genmap();
	}
	else
		isok = false;
	return isok;
}

int usertype::find(const string &mail)
{
	db::result res(database.query("select * from users where email='" + mail + "'"));
	if(!res.ok())return -1;
	
	email = res["email"];
	passwd= res["passwd"];
	
	return 0;
}

int cidtype::find(const string &c)
{
	db::result res(database.query("select * from cids where cid='" + c + "'"));
	if(!res.ok())return -1;
	
	cid      = res["cid"];
	email    = res["email"];
	belltype = atoi(res["belltype"].c_str());
	bell     = res["bell"];
		
	return 0;
}

static string stringize(unsigned n)
{
	char Buf[64];
	sprintf(Buf, "%u", n);
	return Buf;
}

static string escaped(const string &s)
{
	string t;
	
	unsigned a, b=s.size();
	
	for(a=0; a<b; ++a)
	{
		if(s[a] == '\0') t += "\\0";
		else if(s[a] == '\'') t += "\\'";
		else if(s[a] == '\\') t += "\\\\";
		else if(s[a] == '"')  t += "\\\"";
		else t += s[a];
	}	
	
	return t;
}

int message::insert()
{
//	cerr << "senttime(3) = " << senttime << endl;
	
	string SQL = 
		"insert into messages (email,cid,msn,recordtime,perishtime,senttime,"
		                     "pathname,bytelength,timelength,compressiontype,"
		                     "mimetype,voicedata,numberdata) "
		           "values('"+email+"','"+cid+
		                "','"+msn+"',"+stringize(recordtime)+","+stringize(perishtime)
		                          +","+stringize(senttime)+",'"+pathname+
		                          "',"+stringize(bytelength)+","+stringize(timelength)
		                         +",'"+compressiontype+"','"+mimetype
		                        +"','"+escaped(voicedata)+"','"+numberdata+"')";
	
	db::result r = database.query(SQL);
	
//	cerr << "hirvitys = \"" << SQL << "\"\n";
	
	return r.res != NULL;
}
