#ifndef voicebox_db_hh
#define voicebox_db_hh

#include <string>
#include <vector>
#include <ctime>
#include <map>

#include <mysql/mysql.h>

#include "sizes.h"
#include "autoptr.h"

using namespace std;

class dbrow
{
protected:
	class db &database;
public:
	dbrow(class db &d) : database(d) { }
};

class usertype : public dbrow
{
public:
	string email;
	string passwd;
	
	int find(const string &mail);
	usertype(class db &d) : dbrow(d) { }
};

class cidtype : public dbrow
{
public:
	string cid;
	string email;
	int belltype;
	string bell;
	
	int find(const string &c);
	cidtype(class db &d) : dbrow(d) { }
};

class db
{
	MYSQL sql;
	int status;
public:
	typedef map<string, string> rowtype;
	
	class result : public rowtype
	{
		friend class db;
		friend class message;
		int *refcount;
		MYSQL_ROW row;
		MYSQL_RES *res;
		unsigned long *lens;
		unsigned fieldcount;
		vector<string> fields;
		bool isok;
		void genmap();
		result(MYSQL_RES *r);
	public:
		result(const result &r);
		result() : refcount(NULL), res(NULL), isok(false) { }
		~result();
		bool ok() const { return isok; }
		bool movenext();
	};
	
	db();
	~db();
	
	db(const string &host,
	   const string &user,
	   const string &pass);
	
	void connect(const string &host,
	             const string &user,
	             const string &pass);
	
	result query(const string &s);
};

class message : public dbrow
{
public:
	string email;
	string cid;
	string msn;
	time_t recordtime;
	time_t perishtime;
	time_t senttime;
	string pathname;
	unsigned bytelength;
	unsigned timelength;
	string compressiontype;
	string mimetype;
	string voicedata;
	string numberdata;
	
	message(class db &d) : dbrow(d)
	{
		recordtime = perishtime = senttime = 0;
		bytelength = timelength = 0;
		
//		cerr << "senttime(1) = " << senttime << endl;
	}	
	int insert();
};

#endif
