#include <iostream>
#include <string>
#include <cstdio>
#include <unistd.h>

#include "db.hh"
#include "inetstream.hh"

#include "config.h"

// The base64 encoding routine is mainly taken from PHP4.

static char base64_table[] =
{ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
  'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
  'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
  'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '\0'
};
static char base64_pad = '=';

static string base64encode(const string &s)
{
    const unsigned char *current = (const unsigned char *)s.c_str();
    unsigned length = s.size();
    unsigned size = 0;
    
    string result;

    while(length > 2)
    { 
        // keep going until we have less than 24 bits
        result += base64_table[current[0] >> 2];
        result += base64_table[((current[0] & 0x03) << 4) + (current[1] >> 4)];
        result += base64_table[((current[1] & 0x0f) << 2) + (current[2] >> 6)];
        result += base64_table[current[2] & 0x3f];
        
        if((size += 4) >= 60)
        {
        	size = 0;
        	result += "\n";
        }

        current += 3;
        length -= 3; // we just handle 3 octets of data
    }

    // now deal with the tail end of things
    if(length != 0)
    {
        result += base64_table[current[0] >> 2];
        if (length > 1)
        {
            result += base64_table[((current[0] & 0x03) << 4) + (current[1] >> 4)];
            result += base64_table[(current[1] & 0x0f) << 2];
            result += base64_pad;
        }
        else
        {
            result += base64_table[(current[0] & 0x03) << 4];
            result += base64_pad;
            result += base64_pad;
        }
    }    
    return result;
}

int main(void)
{
	db d1(SQL_HOST, SQL_USER, SQL_PASS);
	d1.query("use voicebox");
	
	cerr << "mailer: Mailer task started\n";
	
	for(;;)
	{
    	db::result res(d1.query("select * from messages where senttime=0"));
    	
    	while(res.ok())
    	{
    		// winnie:smtp
    		
    		inetstream tmp(SMTP_HOST, SMTP_PORT);
    		
    		string bound = GENERIC_MIME_BOUNDARY;
    		
    		time_t starttime = atoi(res["recordtime"].c_str());
    		
    		cerr << "mailer: Sending mail for " << res["email"] << endl;
    		
    		string s;
    		
    		for(;;)
    		{
	    		tmp.getline(s);
	    		// greeting...
	    		if(s.substr(0, 4) == "220 ")break;
	   		}
	   		
    		tmp.sendah(
    			"HELO " THISHOST "\n"
    			"MAIL FROM:<" SENDER ">\n"
    			"RCPT TO:<" + res["email"] + ">\n"
    			"DATA\n");
    			
    		for(;;)
    		{
	    		tmp.getline(s);
	    		// give the mail -prompt...
	    		if(s.substr(0, 4) == "354 ")break;
	    	}
	    	
    		tmp.sendah(
    			"Subject: Voicebox entry from number " + res["cid"] + " " + ctime(&starttime) +
    			// ctime makes linefeed automatically.
    	        "X-Mailer: " X_MAILER "\n"
    	        "MIME-Version: 1.0\n"
    	        "Content-Type: MULTIPART/Mixed; BOUNDARY=\""+bound+"\"\n"
    	        "\n"
                "  This message is in MIME format.  The first part should be readable text,\n"
    			"  while the remaining parts are likely unreadable without MIME-aware tools.\n"
    			"  Send mail to mime@docserver.cac.washington.edu for more info.\n"
    			"\n"
    			"--" + bound + "\n"
    		    "Content-Type: TEXT/PLAIN; charset=US-ASCII\n"
    		    "\n");
    		 
    		if(res["numberdata"].size())
    		{
    		    tmp.sendah("The phone number keys you pressed while recording: ");
	    		tmp.sendah(res["numberdata"]);
	    		tmp.sendah("\n\n");
	    	}
    		 
    		tmp.sendah(
    		    "The attached file is a raw 8-bit ulaw 8kHz audio file.\n"
    		    "You can play it like this: cat message.au >/dev/audio\n"
    		    "\n"
    		    "The message file is also located in\n"+
    		    res["pathname"]+" at the Voicebox server.\n"
    		    "\n"
    		    "--" + bound + "\n"
    		    "Content-Type: " + res["mimetype"] + "; name=\"message.au\"\n"
    		    "Content-Transfer-Encoding: BASE64\n"
    		    "Content-Description: The message.\n"
    		    "Content-Disposition: attachment; filename=\"message.au\"\n"
    		    "\n");
    		tmp.sendah(
    		    base64encode(res["voicedata"]));
    		tmp.sendah(
    			"\n"
    		    "--" + bound + "\n"
    		    ".\n");
    		    
    		for(;;)
    		{
	    		tmp.getline(s);
	    		// mail sent...
	    		if(s.substr(0, 4) == "250 ")break;
	   		}
	   		
	   		tmp.sendah("QUIT\n");
    		    
    		for(;;)
    		{
	    		tmp.getline(s);
	    		// byebye...
	    		if(s.substr(0, 4) == "221 ")break;
	   		}
	   		
	   		cerr << "mailer: Mail was sent.\n";
	   		
    		char Buf[256];
    		
			db d2(SQL_HOST, SQL_USER, SQL_PASS);
			d2.query("use voicebox");
    		sprintf(Buf, "update messages set senttime = %ld where id = ", time(0));
    		d2.query(Buf + res["id"]);
    		
    		res.movenext();
    	}
    	sleep(10);
	}
	
	return 0;
}
